using Grpc.Core;
using GrpcService;

namespace GrpcService.Services
{
    public class GreeterService : Greeter.GreeterBase
    {
        private readonly ILogger<GreeterService> _logger;
        public GreeterService(ILogger<GreeterService> logger)
        {
            _logger = logger;
        }

        public override Task<JResponse> SendJoke(JRequest request, ServerCallContext context)
        {
            List<Joke> jokeList = JokeRepo();

            JResponse jRes = new JResponse();
            jRes.Joke.AddRange(jokeList.Skip(request.No - 1).Take(1));

            return Task.FromResult(jRes);
        }

        public List<Joke> JokeRepo()
        {
            List<Joke> jokeList = new List<Joke>
            {
                new Joke { Author = "Random", Description = "I ate a clock yesterday, it was very time-consuming"},
                new Joke { Author = "Xeno", Description = "Have you played the updated kids' game? I Spy With My Little Eye ... Phone"},
                new Joke { Author = "Jak", Description = "A perfectionist walked into a bar...apparently, the bar wasnt set high enough"},
                new Joke { Author = "Peta", Description = "To be or not to be a horse rider, that is equestrian"},
                new Joke { Author = "Katnis", Description = "What does a clam do on his birthday? He shellabrates"}
            };

            return jokeList;
        }

        public override async Task SendJokeSS(JRequest request, IServerStreamWriter<JResponse> responseStream, ServerCallContext context)
        {
            List<Joke> jokeList = JokeRepo();
            JResponse jRes;

            var i = 0;
            while (!context.CancellationToken.IsCancellationRequested)
            {
                jRes = new JResponse();
                jRes.Joke.Add(jokeList.Skip(i).Take(request.No));
                await responseStream.WriteAsync(jRes);
                i++;
                // Gotta look busy (not needed in production)
                await Task.Delay(1000);
            }
        }

        public override async Task<JResponse> SendJokesCS(IAsyncStreamReader<JRequest> requestStream, ServerCallContext context)
        {
            List<Joke> jokeList = JokeRepo();
            JResponse jRes = new JResponse();

            await foreach (var message in requestStream.ReadAllAsync())
            {
                jRes.Joke.Add(jokeList.Skip(message.No - 1).Take(1));
            }
            return jRes;
        }

        public override async Task SendJokesBD(IAsyncStreamReader<JRequest> requestStream, IServerStreamWriter<JResponse> responseStream, ServerCallContext context)
        {
            List<Joke> jokeList = JokeRepo();
            JResponse jRes;

            await foreach (var message in requestStream.ReadAllAsync())
            {
                jRes = new JResponse();
                jRes.Joke.Add(jokeList.Skip(message.No - 1).Take(1));
                await responseStream.WriteAsync(jRes);
            }
        }
    }
}